\name{nclust.mmat}
\alias{nclust.mmat}
\title{nclust for multiple data matrix}
\description{Fast hierarchical cluster for multiple data matrices,
where the columns are assumed to be the objects to be clustered.
}
\usage{
nclust(...,
  distance=c("nzcor.rema","nzcor.fema"),
  link = c("ave","mcquitty","ward","single","complete"),
  branchflip = c("nnephew","tightleft")
  )
}
\arguments{
  \item{...}{Data matrices, with columns as objects
  to be clustered. Column names must be unique and assumed
  to refer to the same object in different matrices. Some objects
  may be missing entirely from some matrices (the number of
  columns do not have to be the same, and can be arbitrarily ordered).
  The result is a hierarchical
  cluster of the union of the column objects.
  }
  \item{distance}{\code{nzcor.rema} is the negative of Fisher
  z-transformed (a.k.a. inverse hyperbolic tangent transformed) Pearson's
  correlation, combined across the matrices using random effects
  meta-analysis. \code{nzcor.fema} is similar, but uses fixed
  effects meta-analysis.}
  \item{link}{Link function between clusters. These are a subset
  of those in \link{hclust} that satisfy the
  reducibility property and amenable to reciprocal nearest-neighbor chain
  algorithm.}
  \item{branchflip}{The default is flip the branches such that
  the nearest nephew is placed close to its uncle. \code{tightleft}
  puts the tighter branch on the left, identical to that of \link{hclust}}
}
\details{
}
\value{
  A list with similar content to the output of \link{hclust}. The
  objects \code{merge}, \code{order}, \code{labels}, and \code{height}
  defines the dendrogram and have the same format as those produced
  by \code{hclust} or \code{nclust}. Additional objects \code{linkscore},
  \code{branch.level} and \code{leaf.level} are the same as those
  produced by \code{nclust}. New objects produced
  by \code{nclust.mmat} are: \code{N} (the number
  of the union of columns), \code{s2} (pooled column variance for
  each object in the union),
  \code{df} (pooled degrees-of-freedom corresponding to \code{s2})
  and \code{unicol}, a matrix of indices that map the union of
  columns to the column in each input matrix (with zero signifies
  missing column).
  
  The tree plotting function \link{plclust} is applicable
  to this output.  
  For a large tree, the functions
  \link{plot.nclust} and \link{dendrolines}
  are more appropriate and flexible.


}
\references{
Benzecri, J. P. (1982)
Construction d'une classification ascendante hierarchique
par la recherce en chaine des voisins reciproques.
Les Cahiers de l'Analyse des Donnees 7:209-218

Bruynooghe, M (1977)
Methodes nouvelles en classification automatique des donnees
taxinomiques nombreuses.
Statistique et Analyse des Donnees 3:24-42.

Gordon A.D. (1999) Classification. 2nd ed. Chapman and Hall/CRC.
}

\author{ Pratyaksha Wirapati }

\seealso{dendrolines, plot.nclust, nclust, nclust2, hclust, plclust}

\examples{
  # typically not used directly
  # see the R code for `nclust2`
}
